# Solitude is a interactive story with four endings.
# Copyright (C) 2012 Ryan Martin
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import pyglet
from pyglet import sprite, image

import Interface

class Character(pyglet.sprite.Sprite):
    def __init__(self, walkcycle_image, **kwargs):
        self.load_walkcycle(walkcycle_image)
        pyglet.sprite.Sprite.__init__(self, self.idle['down'], **kwargs)
        self.set_idle('down')

    def load_walkcycle(self, im):
        walkcyclegrid=image.TextureGrid(image.ImageGrid(im, 4, 1))
        self.walkcycle={}
        self.idle={}
        duration=0.1
        for i, d in enumerate(["right", "down", "left", "up"]):
            cycle=image.ImageGrid(walkcyclegrid[i], 1, 9)
            for frame in cycle:
                frame.anchor_x=frame.width/2
            self.walkcycle[d]=image.Animation(map(lambda f: image.AnimationFrame(f, duration), cycle[1:]))
            self.idle[d]=cycle[0]

    def action(self):
        Interface.dialog_text.text="Character Action"
        Interface.show_dialog()

    def set_walkcycle(self, direction):
        self.image=self.walkcycle[direction]
        self.direction=direction

    def set_idle(self, direction):
        self.image=self.idle[direction]
        self.direction=direction
