# Solitude is a interactive story with four endings.
# Copyright (C) 2012 Ryan Martin
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os.path
import sys
import json

import pyglet
from pyglet.gl import *
from pyglet import sprite
from pyglet import image
from pyglet import resource
from Viewport import FixedResolutionViewport

import GameData
import Interface
from Character import Character
from Entity import Entity
from Stage import Stage

class City(Stage):
    def __init__(self):
        width, height=self.load_map("city.json")
        Stage.__init__(self, width, height)
        self.background=sprite.Sprite(resource.image("city_background.png"), batch=self.batch, group=self.backgroup)
        self.building=self.new_sprite(5, 18, resource.image("city_building1.png"))
        self.building.visible=False
        self.player.load_walkcycle(resource.image("city_walkcycle.png"))
        self.player.set_idle("down")
        self.player.x=15*32+16
        self.player.y=35*32+16
        self.set_group(self.player)
        self.player.set_idle('up')
        self.update_camera()
        for i in range(13,17):
            self.set_tile_action(i, 18, self.hide_building)
            self.set_tile_action(i, 17, self.show_building)
        self.hotdog_cart=self.new_sprite(50, 9, resource.image("hotdog_cart.png"))
        self.richard=Entity(resource.image("richard.png"), batch=self.batch, group=self.ogroups[1])
        self.richard.x=52*32
        self.richard.y=9*32
        self.richard.action=lambda text="I'm just doing this for research.": self.show_dialog(text)
        self.set_group(self.richard)
        self.add_entity(self.richard)
        tree_image=resource.image("tree.png")
        tree_image.anchor_x=tree_image.width/2
        self.trees=[]
        for i,j in self.tree_locations:
            tree=self.new_sprite(i, j, tree_image)
            tree.x+=16
            self.trees.append(tree)
        boss_walkcycle=image.TextureGrid(image.ImageGrid(resource.image("agent.png"), 4, 9))
        self.boss=Entity(boss_walkcycle[(1,0)], batch=self.batch, group=self.ogroups[1])
        self.boss.image.anchor_x=self.boss.width/2
        self.boss.x=15*32+16
        self.boss.y=38*32
        self.boss.action=lambda text="Deliver report?", action=self.deliver_report: self.show_yes_no_dialog(text, action)
        self.set_group(self.boss)
        self.add_entity(self.boss)
        coworker_walkcycle=image.TextureGrid(image.ImageGrid(resource.image("coworker.png"), 4, 9))
        self.coworkers=[]
        self.money=[500, 300, 750, 425]
        for n, (i,j) in enumerate(self.coworker_locations):
            if i==19:
                coworker=Entity(coworker_walkcycle[(2,0)], batch=self.batch, group=self.ogroups[1])
            else:
                coworker=Entity(coworker_walkcycle[(0,0)], batch=self.batch, group=self.ogroups[1])
            coworker.image.anchor_x=coworker.width/2
            coworker.x=i*32+16
            coworker.y=j*32
            coworker.action=lambda m=self.money[n]: self.show_dialog("I spent $%d this month." % m)
            self.set_group(coworker)
            self.add_entity(coworker)
            self.coworkers.append(coworker)
        self.set_action(7*32, 39*32, lambda: self.show_dialog(["Still hasn't been refilled.", "Even if it were, the heating element broke last month."]))
        self.set_action(8*32, 39*32, lambda: self.show_dialog(["Still hasn't been refilled.", "Even if it were, the heating element broke last month."]))
        professor_walkcycle=image.TextureGrid(image.ImageGrid(resource.image("professor.png"), 4, 9))
        self.professor=Entity(professor_walkcycle[(2,0)], batch=self.batch, group=self.ogroups[1])
        self.professor.image.anchor_x=self.professor.width/2
        self.professor.x=48*32+16
        self.professor.y=29*32+16
        self.professor.action=self.talk_professor
        self.set_group(self.professor)
        self.add_entity(self.professor)
        maskman_walkcycle=image.TextureGrid(image.ImageGrid(resource.image("maskman.png"), 4, 9))
        self.maskman=Entity(maskman_walkcycle[(0,0)], batch=self.batch, group=self.ogroups[1])
        self.maskman.image.anchor_x=self.maskman.width/2
        self.maskman.x=37*32+16
        self.maskman.y=13*32
        self.maskman.action=lambda: self.show_dialog(["Man it's hot.", "This suit is killing me."])
        self.set_group(self.maskman)
        self.add_entity(self.maskman)
        self.balloons=self.new_sprite(37, 12, resource.image("balloons.png"))
        self.balloons.x-=8
        self.balloons.y+=16
        self.set_collision(37*32, 12*32)
        mother_walkcycle=image.TextureGrid(image.ImageGrid(resource.image("mother.png"), 4, 9))
        self.mother=Entity(mother_walkcycle[(2,0)], batch=self.batch, group=self.ogroups[1])
        self.mother.image.anchor_x=self.mother.width/2
        self.mother.x=38*32+16
        self.mother.y=13*32+16
        self.mother.action=lambda: self.show_dialog("Hold on sweetie, Mommy has to find her wallet.")
        self.set_group(self.mother)
        self.add_entity(self.mother)
        child_walkcycle=image.TextureGrid(image.ImageGrid(resource.image("child.png"), 4, 4))
        self.child=Entity(child_walkcycle[(1,1)], batch=self.batch, group=self.ogroups[1])
        self.child.image.anchor_x=self.child.width/2
        self.child.x=38*32+16
        self.child.y=12*32+16
        self.child.action=lambda: self.show_dialog("I want the red one. The red one!")
        self.set_group(self.child)
        self.add_entity(self.child)
        self.find_liar=False
        self.is_dream=True
        if Interface.music:
            self.music=resource.media("city.ogg")

    def display(self, music):
        Stage.display(self, music)
        self.fade_in(lambda text=["I need you to get the expense figures from all your coworkers.","When you are done, get back to me with a report."]:
                        self.show_dialog(text))
        #self.show_dialog(["I need you to get the expense figures from all your coworkers.","When you are done, put it in a report and give it to me."])

    def talk_professor(self):
        self.show_dialog(["I'm a professor at the local university.", "I study microexpressions and how they relate to lying.",
                          "Let me give you a short dissertation on the subject...", "...", "... ...", "... ... ...",
                          "...and that's the current state of research on microexpressions."])
        for n,coworker in enumerate(self.coworkers):
            i=int(coworker.x/32)
            j=int(coworker.y/32)
            if i==11 and j==26:
                self.set_action(coworker.x, coworker.y, lambda m=self.money[n]: self.show_dialog("I spent $%d this month. (Lie)" % m, self.toggle_liar))
            else:
                self.set_action(coworker.x, coworker.y, lambda m=self.money[n]: self.show_dialog("I spent $%d this month. (True)" % m))

    def toggle_liar(self):
        self.find_liar=True

    def deliver_report(self):
        if self.find_liar:
            self.show_dialog("Good job.  I suspected someone was lying.", lambda: Interface.switch_stage("Mansion"))
        else:
            self.show_dialog("Took you long enough.  Now get back to work.", lambda: Interface.switch_stage("Cubefarm"))

    def show_building(self):
        self.building.visible=True

    def hide_building(self):
        self.building.visible=False

    def load_map(self, mapname):
        width, height=Stage.load_map(self, mapname)
        for layer in self.layers:
            if layer["name"]=="Trees":
                self.tree_locations=self.get_locations(layer['data'], width, height)
            if layer["name"]=="Coworkers":
                self.coworker_locations=self.get_locations(layer['data'], width, height)
        return width, height

