# Solitude is a interactive story with four endings.
# Copyright (C) 2012 Ryan Martin
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os.path
import sys
import json

import pyglet
from pyglet.gl import *
from pyglet import sprite
from pyglet import image
from pyglet import resource

from Viewport import FixedResolutionViewport

import GameData
import Interface
from Character import Character
from Entity import Entity
from Stage import Stage

class Cubefarm(Stage):
    def __init__(self):
        width, height=self.load_map("cubefarm.json")
        Stage.__init__(self, width, height)
        self.background=sprite.Sprite(resource.image("cubefarm_background.png"), batch=self.batch, group=self.backgroup)
        self.player.load_walkcycle(resource.image("city_walkcycle.png"))
        self.player.set_idle("down")
        self.player.x=13*32
        self.player.y=15*32
        self.set_group(self.player)
        self.update_camera()
        boss_walkcycle=image.TextureGrid(image.ImageGrid(resource.image("agent.png"), 4, 9))
        self.boss=self.new_sprite(10, 20, boss_walkcycle[(1,0)])
        self.boss.x-=16
        self.boss.y+=16
        self.set_action(10*32, 20*32, self.boss_talk)
        coworker_walkcycle=image.TextureGrid(image.ImageGrid(resource.image("coworker.png"), 4, 9))
        self.coworkers=[]
        for i,j in self.coworker_locations:
            if i==14:
                coworker=self.new_sprite(i, j, coworker_walkcycle[(2,0)])
            else:
                coworker=self.new_sprite(i, j, coworker_walkcycle[(0,0)])
            coworker.x-=16
            coworker.y+=16
            self.coworkers.append(coworker)
            self.set_action(i*32, j*32, self.coworker_talk)
        self.set_tile_action(8, 0, self.end_stage)
        self.set_tile_action(9, 0, self.end_stage)
        self.set_tile_action(10, 0, self.end_stage)
        self.set_tile_action(11, 0, self.end_stage)

        self.boss_cursor=0
        self.boss_text=[["Where is the report I asked for?"],
                    ["Worthless.", "You can't even make coffee right."],
                    ["Ahh, I'm also gonna need you to go ahead and come in on Sunday, too..."]]
        self.coworker_cursor=0
        self.coworker_text=[["Haven't gotten a raise in two years."],
                    ["Don't tell anyone but I've got a line on another job.", "It'll be hell of a lot better than this one."],
                    ["The boss said I could have this weekend off if I give him three more TPS reports today."],
                    ["I am going to get so wasted this weekend."],
                    ["Anyone seen my stapler?"]]
        self.is_dream=True
        if Interface.music:
            self.music=resource.media("cubefarm.ogg")

    def get_boss_text(self):
        text=self.boss_text[self.boss_cursor]
        self.boss_cursor+=1
        if self.boss_cursor>=len(self.boss_text):
            self.boss_cursor=0
        return text

    def boss_talk(self):
        text=self.get_boss_text()
        self.show_dialog(text)

    def get_coworker_text(self):
        text=self.coworker_text[self.coworker_cursor]
        self.coworker_cursor+=1
        if self.coworker_cursor>=len(self.coworker_text):
            self.coworker_cursor=0
        return text

    def coworker_talk(self):
        text=self.get_coworker_text()
        self.show_dialog(text)

    def end_stage(self):
        self.fade_out(lambda: self.show_dialog(["I didn't fit in that everyday world.", "So I walked away."], lambda: Interface.switch_stage("Credits")))

    def load_map(self, mapname):
        width, height=Stage.load_map(self, mapname)
        for layer in self.layers:
            if layer["name"]=="Coworkers":
                self.coworker_locations=self.get_locations(layer['data'], width, height)
        return width, height
