# Solitude is a interactive story with four endings.
# Copyright (C) 2012 Ryan Martin
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os.path
import sys
import json

import pyglet
from pyglet.gl import *
from pyglet import sprite
from pyglet import image
from pyglet import resource

from Viewport import FixedResolutionViewport

import GameData
import Interface
from Character import Character
from Entity import Entity
from Stage import Stage

class Farm(Stage):
    def __init__(self):
        width, height=self.load_map("farm.json")
        Stage.__init__(self, width, height)
        self.background=sprite.Sprite(resource.image("farm_background.png"), batch=self.batch, group=self.backgroup)
        self.building1=sprite.Sprite(resource.image("farm_building1.png"), batch=self.batch, group=self.ogroups[1])
        self.building1.image.anchor_y=8
        self.building1.x=7*32
        self.building1.y=19*32
        self.hide_building1()
        self.set_group(self.building1)
        self.building2=sprite.Sprite(resource.image("farm_building1.png"), batch=self.batch, group=self.ogroups[1])
        self.building2.image.anchor_y=8
        self.building2.x=51*32
        self.building2.y=28*32
        self.set_group(self.building2)
        self.player.load_walkcycle(resource.image("farm_walkcycle.png"))
        self.player.set_idle("down")
        self.player.x=17*32 
        self.player.y=26*32 
        self.set_group(self.player)
        self.update_camera()
        self.stove=sprite.Sprite(resource.image("stove_fire.png"), batch=self.batch, group=self.ogroups[1])
        self.stove.x=9*32
        self.stove.y=25*32
        self.set_group(self.stove)
        tree_image=resource.image("pine_tree.png")
        tree_image.anchor_x=tree_image.width/2
        self.trees=[]
        for i,j in self.tree_locations:
            tree=sprite.Sprite(tree_image, batch=self.batch, group=self.ogroups[1])
            tree.x=i*32+16
            tree.y=j*32
            self.set_group(tree)
            self.trees.append(tree)
        self.set_tile_action(12, 19, self.hide_building1)
        self.set_tile_action(12, 18, self.show_building1)
        self.ring=self.flowers=self.peppers=False
        for i,j in self.ring_locations:
            self.set_action(i*32, j*32, lambda text="Take ring?", action=self.take_ring: self.show_yes_no_dialog(text, action))
        for i,j in self.flower_locations:
            self.set_action(i*32, j*32, lambda text="Pick flowers?", action=self.pick_flowers: self.show_yes_no_dialog(text, action))
        for i,j in self.pepper_locations:
            self.set_action(i*32, j*32, lambda text="Harvest peppers?", action=self.harvest_peppers: self.show_yes_no_dialog(text, action))
        for i,j in self.bookcase_locations:
            self.set_action(i*32, j*32, self.read_diary)
        self.set_action(78*32, 3*32, lambda: self.show_dialog("It hasn't moved since I pushed it over years ago."))
        female_walkcycle=image.TextureGrid(image.ImageGrid(resource.image("astraea.png"), 4, 9))
        self.woman=Entity(female_walkcycle[(1,0)], batch=self.batch, group=self.ogroups[1])
        self.woman.image.anchor_x=self.woman.width/2
        self.woman.x=61*32+16
        self.woman.y=46*32
        self.woman.action=lambda text="Propose?", action=self.propose: self.show_yes_no_dialog(text, action)
        self.set_group(self.woman)
        self.add_entity(self.woman)
        self.diary_cursor=0
        self.diary=[["A girl moved into the old Johnson place today.", "I hope she doesn't try and bother me and Jake"],
                    ["Jake and me hid in the corn fields when the girl followed us today.", "Astraea is a weird name."],
                    ["The funeral for Jake's mom was today.", "Last time I saw him cry was when he fell out of a tree and broke his arm."],
                    ["Jake and his dad left for the city this morning.", "He said he'd be back this summer."],
                    ["Got a letter from Jake.", "Said his first period class has more people than our entire school."],
                    ["Went swimming with Astraea today.", "She said this is her favorite season, on account of the flowers that grow next to the pond."],
                    ["Got another letter from Jake, first one in months.", "Said his dad got another job so they can't come down for the summer."],
                    ["Had an eating contest with Astraea, didn't end well.", "That girl loves spicy food."],
                    ["Dad gave me a box with a ring in it.", "Said it was my grandmother's and I should keep it safe for when I needed it.", "Not sure what he means."],
                    ["Astraea bet me I couldn't move one of those big rocks over to the east.", "Boy was she surprised when I rolled it right over."],
                    ["I got a book about Asia from Astraea today.", "She's crazy about that stuff."],
                    ["Jake came back yesterday.", "It's good to see him but I don't like the way he looks at Astraea."],
                    ["Jake finally left today.", "I hope Astraea will stop mentioning him soon."]]
        self.is_dream=True
        if Interface.music:
            self.music=resource.media("farm.ogg")

    def display(self, music):
        Stage.display(self, music)
        self.fade_in()

    def get_diary_text(self):
        text=self.diary[self.diary_cursor]
        self.diary_cursor+=1
        if self.diary_cursor>=len(self.diary):
            self.diary_cursor=0
        return text

    def take_ring(self):
        self.ring=True

    def pick_flowers(self):
        self.flowers=True
    
    def harvest_peppers(self):
        self.peppers=True

    def propose(self):
        if self.ring and self.flowers and self.peppers:
            Interface.switch_stage("Shrine")
        else:
            Interface.switch_stage("Farmhouse")

    def read_diary(self):
        text=self.get_diary_text()
        self.show_dialog(text)

    def load_map(self, mapname):
        width, height=Stage.load_map(self, mapname)
        for layer in self.layers:
            if layer["name"]=="Trees":
                self.tree_locations=self.get_locations(layer['data'], width, height)
            if layer["name"]=="Bookcase":
                self.bookcase_locations=self.get_locations(layer['data'], width, height)
            if layer["name"]=="Ring":
                self.ring_locations=self.get_locations(layer['data'], width, height)
            if layer["name"]=="Flowers":
                self.flower_locations=self.get_locations(layer['data'], width, height)
            if layer["name"]=="Peppers":
                self.pepper_locations=self.get_locations(layer['data'], width, height)
        return width, height

    def show_building1(self):
        self.building1.visible=True

    def hide_building1(self):
        self.building1.visible=False

